@@ -21,6 +21,9 @@ import warlock
 from glanceclient.v2 import images
 from tests import utils
 
+_CHKSUM = '93264c3edf5972c9f1cb309543d38a5c'
+_CHKSUM1 = '54264c3edf5972c9f1cb309453d38a46'
+
 _BOGUS_ID = '63e7f218-29de-4477-abdc-8db7c9533188'
 _EVERYTHING_ID = '802cbbb7-0379-4c38-853f-37302b5e3d29'
 _OWNED_IMAGE_ID = 'a4963502-acc7-42ba-ad60-5aa0962b7faf'
@@ -85,6 +88,15 @@ fixtures = {
             '',
         ),
     },
+    '/v2/images': {
+        'POST': (
+            {},
+            {
+                'id': '3a4560a1-e585-443e-9b39-553b46ec92d1',
+                'name': 'image-1',
+            },
+        ),
+    },
     'v2/images/87b634c1-f893-33c9-28a9-e5673c99239a': {
         'DELETE': (
             {},
@@ -93,6 +105,12 @@ fixtures = {
             },
         ),
     },
+    '/v2/images/606b0e88-7c5a-4d54-b5bb-046105d4de6f/file': {
+        'PUT': (
+            {},
+            '',
+        ),
+    },
     '/v2/images/5cc4bebc-db27-11e1-a1eb-080027cbe205/file': {
         'GET': (
             {},
@@ -189,6 +207,38 @@ fixtures = {
             ]},
         ),
     },
+    '/v2/images?checksum=%s&limit=%d' % (_CHKSUM, images.DEFAULT_PAGE_SIZE): {
+        'GET': (
+            {},
+            {'images': [
+                {
+                    'id': '3a4560a1-e585-443e-9b39-553b46ec92d1',
+                    'name': 'image-1',
+                }
+            ]},
+        ),
+    },
+    '/v2/images?checksum=%s&limit=%d' % (_CHKSUM1, images.DEFAULT_PAGE_SIZE): {
+        'GET': (
+            {},
+            {'images': [
+                {
+                    'id': '2a4560b2-e585-443e-9b39-553b46ec92d1',
+                    'name': 'image-1',
+                },
+                {
+                    'id': '6f99bf80-2ee6-47cf-acfe-1f1fabb7e810',
+                    'name': 'image-2',
+                },
+            ]},
+        ),
+    },
+    '/v2/images?checksum=wrong&limit=%d' % images.DEFAULT_PAGE_SIZE: {
+        'GET': (
+            {},
+            {'images': []},
+        ),
+    },
 }
 
 
@@ -243,6 +293,27 @@ class TestController(testtools.TestCase):
         images = list(self.controller.list(**filters))
         self.assertEqual(images[0].id, _OWNED_IMAGE_ID)
 
+    def test_list_images_for_checksum_single_image(self):
+        fake_id = '3a4560a1-e585-443e-9b39-553b46ec92d1'
+        filters = {'filters': dict([('checksum', _CHKSUM)])}
+        images = list(self.controller.list(**filters))
+        self.assertEquals(1, len(images))
+        self.assertEqual(images[0].id, '%s' % fake_id)
+
+    def test_list_images_for_checksum_multiple_images(self):
+        fake_id1 = '2a4560b2-e585-443e-9b39-553b46ec92d1'
+        fake_id2 = '6f99bf80-2ee6-47cf-acfe-1f1fabb7e810'
+        filters = {'filters': dict([('checksum', _CHKSUM1)])}
+        images = list(self.controller.list(**filters))
+        self.assertEquals(2, len(images))
+        self.assertEqual(images[0].id, '%s' % fake_id1)
+        self.assertEqual(images[1].id, '%s' % fake_id2)
+
+    def test_list_images_for_wrong_checksum(self):
+        filters = {'filters': dict([('checksum', 'wrong')])}
+        images = list(self.controller.list(**filters))
+        self.assertEquals(0, len(images))
+
     def test_list_images_for_bogus_owner(self):
         filters = {'filters': dict([('owner', _BOGUS_ID)])}
         images = list(self.controller.list(**filters))
@@ -255,11 +326,32 @@ class TestController(testtools.TestCase):
         images = list(self.controller.list(**filters))
         self.assertEqual(images[0].id, _EVERYTHING_ID)
 
+    def test_list_images_filters_encoding(self):
+        filters = {"owner": u"ni\xf1o"}
+        try:
+            list(self.controller.list(filters=filters))
+        except KeyError:
+            # NOTE(flaper87): It raises KeyError because there's
+            # no fixture supporting this query:
+            #   /v2/images?owner=ni%C3%B1o&limit=20
+            # We just want to make sure filters are correctly encoded.
+            pass
+
+        self.assertEqual(filters["owner"], "ni\xc3\xb1o")
+
     def test_get_image(self):
         image = self.controller.get('3a4560a1-e585-443e-9b39-553b46ec92d1')
         self.assertEqual(image.id, '3a4560a1-e585-443e-9b39-553b46ec92d1')
         self.assertEqual(image.name, 'image-1')
 
+    def test_create_image(self):
+        properties = {
+            'name': 'image-1'
+        }
+        image = self.controller.create(**properties)
+        self.assertEqual(image.id, '3a4560a1-e585-443e-9b39-553b46ec92d1')
+        self.assertEqual(image.name, 'image-1')
+
     def test_delete_image(self):
         self.controller.delete('87b634c1-f893-33c9-28a9-e5673c99239a')
         expect = [
@@ -269,6 +361,15 @@ class TestController(testtools.TestCase):
                 None)]
         self.assertEqual(self.api.calls, expect)
 
+    def test_data_upload(self):
+        image_data = 'CCC'
+        image_id = '606b0e88-7c5a-4d54-b5bb-046105d4de6f'
+        self.controller.upload(image_id, image_data)
+        expect = [('PUT', '/v2/images/%s/file' % image_id,
+                  {'Content-Type': 'application/octet-stream'},
+                  image_data)]
+        self.assertEqual(self.api.calls, expect)
+
     def test_data_without_checksum(self):
         body = self.controller.data('5cc4bebc-db27-11e1-a1eb-080027cbe205',
                                     do_checksum=False)
@@ -289,7 +390,7 @@ class TestController(testtools.TestCase):
         try:
             body = ''.join([b for b in body])
             self.fail('data did not raise an error.')
-        except IOError, e:
+        except IOError as e:
             self.assertEqual(errno.EPIPE, e.errno)
             msg = 'was 9d3d9048db16a7eee539e93e3618cbe7 expected wrong'
             self.assertTrue(msg in str(e))
